package tools.descartes.dql.connector.kieker.structures.identifier;





public class ComponentIdentifier extends AbstractIdentifier{
	

	private static final String interfacePattern = "((\\.)(\\w)*[<(].*[>)])";
	
	private String classID;
	private boolean isInterface;
	private String interfaceID;
	public String getComponentID() {
		return classID;
	}

	public boolean isInterfaceIdentifier(){
		return isInterface;
	}

	
	private ComponentIdentifier() {} //Constructor disabled

	public static synchronized ComponentIdentifier createClassComponentIdentifier(String hostname,String classID){ 
		ComponentIdentifier componentIdentifier = new ComponentIdentifier();
		componentIdentifier.isInterface =false;
		componentIdentifier.setHostID(hostname);
		componentIdentifier.classID = classID;
		return componentIdentifier;
	}

	public static synchronized ComponentIdentifier createInterfaceComponentIdentifier(String hostname, String classID, String interfaceID){ 
		ComponentIdentifier componentIdentifier = new ComponentIdentifier();
		componentIdentifier.interfaceID=interfaceID;
		componentIdentifier.isInterface =true;
		componentIdentifier.classID = classID;
		componentIdentifier.setHostID(hostname);
		return componentIdentifier;
	}

	public static synchronized ComponentIdentifier createInterfaceComponentIdentifier(String hostname, String combinedID){ 
		
		String[] combined = combinedID.split(interfacePattern);
		ComponentIdentifier componentIdentifier = new ComponentIdentifier();
		componentIdentifier.interfaceID=combinedID.substring(combined[0].length());
		componentIdentifier.isInterface =true;
		componentIdentifier.classID = combined[0];
		componentIdentifier.setHostID(hostname);
		return componentIdentifier;
	}


	@Override
	public String toString() {
		if (isInterfaceIdentifier()) {
			return super.toString() + "[I] : " + this.classID + "." + this.interfaceID;
		}
		return super.toString() + "[C] : " + this.classID ;
	}


	/* 
	 * !!!
	 * 
	 * Warning: 
	 * For further use it is recommended to exclude interfaceID from hashcode() and equals();
	 * Thus interfaces can be mapped to their parent classes;
	 * 
	 * !!!
	 */
	
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((classID == null) ? 0 : classID.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		ComponentIdentifier other = (ComponentIdentifier) obj;
		if (classID == null) {
			if (other.classID != null)
				return false;
		} else if (!classID.equals(other.classID))
			return false;
		return true;
	}


}
